package com.qmth.exam.reserve.controller.student;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.exam.reserve.bean.Constants;
import com.qmth.exam.reserve.bean.login.LoginReq;
import com.qmth.exam.reserve.bean.login.LoginUser;
import com.qmth.exam.reserve.bean.login.WechatLoginReq;
import com.qmth.exam.reserve.config.WxProperty;
import com.qmth.exam.reserve.controller.BaseController;
import com.qmth.exam.reserve.service.AuthService;
import com.qmth.exam.reserve.weixin.OauthAccessTokenRequest;
import com.qmth.exam.reserve.weixin.response.OauthAccessTokenResponseJson;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

@RestController
@Api(tags = "【考生端】考生登录相关接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/student")
@Aac(strict = false, auth = false)
public class StudentLoginController extends BaseController {

    private static final Logger log = LoggerFactory.getLogger(StudentLoginController.class);

    @Autowired
    private AuthService authService;

    @Autowired
    private WxProperty wxProperty;

    @ApiOperation(value = "考生登录（账号）")
    @PostMapping(value = "/login")
    public LoginUser login(@RequestBody LoginReq req) {
        return authService.studentLogin(req);
    }

    @ApiOperation(value = "考生登录（微信号）")
    @PostMapping(value = "/login/for/wechat")
    public LoginUser loginForWechat(@RequestBody WechatLoginReq req) {
        return authService.wechatLogin(req);
    }

    @Aac(strict = false, auth = true)
    @ApiOperation(value = "考生登出")
    @PostMapping(value = "/logout")
    public void logout() {
        authService.logout(curLoginStudent());
    }

    @ApiOperation(value = "获取用户的openid")
    @PostMapping(value = "/login/get/openid")
    public String getOpenid(@ApiParam("微信平台返回的用户code") @RequestParam String code) {
        OauthAccessTokenRequest tokenRequest = new OauthAccessTokenRequest(code, wxProperty.getAppId(),
                wxProperty.getAppSecret());
        OauthAccessTokenResponseJson responseJson = tokenRequest.request();
        if (responseJson.getErrcode() != 0) {
            log.warn("openId获取失败：用户code：" + code);
            throw new StatusException(Constants.SYSTEM_BUSY);
        }
        return responseJson.getOpenid();
    }

}
